#include <SPI.h>          
#include <I2C.h>                                                                       
#include <NavIMU.h>                    
#include <NavCompass.h>


const float R2D = 57.296;

const int SPI_CS_MPU6000 = 7;


NavIMU     imu(SPI_CS_MPU6000);
NavCompass compass;


//-----------------------------------------------------------------------------
//
void setup()
{
  Serial.begin(57600);

  SPI.begin();
  SPI.setClockDivider(SPI_CLOCK_DIV16);           // 1 MHz rate

  pinMode(SPI_CS_MPU6000, OUTPUT);
  digitalWrite(SPI_CS_MPU6000, HIGH);  delay(1);

  imu.init();

  compass.init();
  compass.set_offsets(-81, -33, -14);

}


//-----------------------------------------------------------------------------
//
void loop()
{
  static unsigned long loopEpoch;
  static unsigned int  loopCount;


  if (millis() - loopEpoch < 9) {
    return;
  }
  loopEpoch = millis();


  float dt;
  float wx, wy, wz;
  float ax, ay, az;
  float mx, my, mz;
  float phi, theta, psi;


  imu.update();

  imu.get_accel(&ax, &ay, &az);
  imu.get_gyro(&wx, &wy, &wz);
  dt = imu.get_sample_time();

  compass.update();
  delay(1);
  compass.get_magnet(&mx, &my, &mz);

  MadgwickAHRS(wx, wy, wz, ax, ay, az, mx, my, mz, dt, &phi, &theta, &psi);
  
  /*
  unsigned int value;
  byte packet[8];

  packet[0] = 0xFF;

  value = (phi*R2D + 180)*10;      // [-180, 180] --> [0, 3600]
  packet[1] = value / 100;
  packet[2] = value - packet[1]*100;

  value = (theta*R2D + 180)*10;
  packet[3] = value / 100;
  packet[4] = value - packet[3]*100;

  value = (psi*R2D + 180)*10;
  packet[5] = value / 100;
  packet[6] = value - packet[5]*100;

  value = packet[1];
  for (int i = 2; i <= 6; i++) {
    value += packet[i];
  }
  packet[7] = value;

  Serial.write((byte *)packet, 8);
  */
  
  loopCount++;
  if (20 <= loopCount) {
    loopCount = 0;

    Serial.print(phi*R2D, 2);    Serial.print(", "); 
    Serial.print(theta*R2D, 2);  Serial.print(", "); 
    Serial.print(psi*R2D, 2);    Serial.println();
  }

}
