float q1 = 0;
float q2 = 0;
float q3 = 0;
float q4 = 1;


//-----------------------------------------------------------------------------
//
void MahonyAHRS(float wx, float wy, float wz, float ax, float ay, float az, float mx, float my, float mz, float dt, float *phi, float *theta, float*psi)
{
  float norm;
  
  float gbx;
  float gby;
  float gbz;

  float mbx;
  float mby;
  float mbz;

  float ex;
  float ey;
  float ez;

  

  norm = sqrt(ax*ax + ay*ay + az*az);
  ax /= norm;
  ay /= norm;
  az /= norm;

  norm = sqrt(mx*mx + my*my + mz*mz);
  mx /= norm;
  my /= norm;
  mz /= norm;
 
  gbx = 2*(q1*q3 - q2*q4);               // g in body axes
  gby = 2*(q2*q3 + q4*q1);
  gbz = 1 - 2*(q1*q1 + q2*q2);

  float hx = mx*(1-2*(q2*q2+q3*q3)) + my*2*(q2*q1-q3*q4)     + mz*2*(q3*q1+q2*q4);
  float hy = mx*2*(q1*q2+q3*q4)     + my*(1-2*(q1*q1+q3*q3)) + mz*2*(q3*q2-q1*q4);
  float hz = mx*2*(q1*q3-q2*q4)     + my*2*(q2*q3+q1*q4)     + mz*(1-2*(q1*q1+q2*q2));
  float bx = sqrt(hx*hx + hy*hy);
  float bz = hz;

  mbx = bx*(1-2*(q2*q2 + q3*q3)) + bz*2*(q1*q3 - q2*q4);
  mby = bx*2*(q2*q1 - q3*q4)     + bz*2*(q2*q3 + q1*q4);
  mbz = bx*2*(q3*q1 + q2*q4)     + bz*(1-2*(q1*q1+q2*q2));

  ex = (ay*gbz - az*gby) - (my*mbz - mz*mby);
  ey = (az*gbx - ax*gbz) - (mz*mbx - mx*mbz);
  ez = (ax*gby - ay*gbx) - (mx*mby - my*mbx);

  wx -= 5*ex;
  wy -= 5*ey;
  wz -= 5*ez;

  q1 += 0.5*( q4*wx - q3*wy + q2*wz)*dt;
  q2 += 0.5*( q3*wx + q4*wy - q1*wz)*dt;
  q3 += 0.5*(-q2*wx + q1*wy + q4*wz)*dt;
  q4 += 0.5*(-q1*wx - q2*wy - q3*wz)*dt;

  norm = sqrt(q1*q1 + q2*q2 + q3*q3 + q4*q4);
  q1 /= norm;
  q2 /= norm;
  q3 /= norm;
  q4 /= norm;

  *phi   =  atan2(2*q2*q3 + 2*q1*q4, 1 - 2*(q1*q1 + q2*q2));
  *theta = -asin(2*q1*q3 - 2*q2*q4);
  *psi   =  atan2(2*q1*q2 + 2*q3*q4, 1 - 2*(q2*q2 + q3*q3));
}

