//-----------------------------------------------------------------------------
//
void EulerCF(float p, float q, float r, float ax, float ay, float dt, float *phi, float *theta)
{
  static float prevPhi;
  static float prevTheta;
  
  float phi_a, theta_a;
  float dotPhi, dotTheta;
  

  EulerAccel(ax, ay, &phi_a, &theta_a);

  BodyToInertial(p, q, r, prevPhi, prevTheta, &dotPhi, &dotTheta);

  *phi   = HpfPhi(dotPhi)     + LpfPhi(phi_a);
  *theta = HpfTheta(dotTheta) + LpfTheta(theta_a);
  
  prevPhi   = *phi;
  prevTheta = *theta;
}


//-----------------------------------------------------------------------------
//
void BodyToInertial(float p, float q, float r, float phi, float theta, float *dotPhi, float *dotTheta)
{
  float sinPhi;
  float cosPhi;
  float tanTheta;
  
  
  sinPhi = sin(phi);
  cosPhi = cos(phi);
  tanTheta = tan(theta);

  *dotPhi   = p + q*sinPhi*tanTheta + r*cosPhi*tanTheta;
  *dotTheta =     q*cosPhi          - r*sinPhi;
}


//-----------------------------------------------------------------------------
//          s^2            1 
//  -----------------  x  ---
//   s^2 + s*kp + ki       s
//
float HpfPhi(float hpf_phi_g)
{
  static float hpf_phi_k_1;
  static float hpf_phi_k_2;
  static float hpf_phi_g_k_1;
  static float hpf_phi_g_k_2;
  
  float hpf_phi;
  
  
  hpf_phi = 1.9972*hpf_phi_k_1 - 0.9972*hpf_phi_k_2 + 0.01*hpf_phi_g - 0.01*hpf_phi_g_k_2;
  
  hpf_phi_k_2 = hpf_phi_k_1;
  hpf_phi_k_1 = hpf_phi;
  
  hpf_phi_g_k_2 = hpf_phi_g_k_1;
  hpf_phi_g_k_1 = hpf_phi_g;
  
  return hpf_phi; 
}


//-----------------------------------------------------------------------------
//      2*kp + ki
//  -----------------
//   s^2 + s*kp + ki
//
float LpfPhi(float phi_a)
{
  static float phi_k_1;
  static float phi_k_2;
  static float phi_a_k_1;
  static float phi_a_k_2;
  
  float phi;
  
  
  phi = 1.9972*phi_k_1 - 0.9972*phi_k_2 + 0.0014*phi_a - 0.0014*phi_a_k_2;
  
  phi_k_2 = phi_k_1;
  phi_k_1 = phi;
  
  phi_a_k_2 = phi_a_k_1;
  phi_a_k_1 = phi_a;
  
  return phi;
}


//-----------------------------------------------------------------------------
//          s^2            1 
//  -----------------  x  ---
//   s^2 + s*kp + ki       s
//
float HpfTheta(float hpf_theta_g)
{
  static float hpf_theta_k_1;
  static float hpf_theta_k_2;
  static float hpf_theta_g_k_1;
  static float hpf_theta_g_k_2;
  
  float hpf_theta;
  
  
  hpf_theta = 1.9972*hpf_theta_k_1 - 0.9972*hpf_theta_k_2 + 0.01*hpf_theta_g - 0.01*hpf_theta_g_k_2;
  
  hpf_theta_k_2 = hpf_theta_k_1;
  hpf_theta_k_1 = hpf_theta;
  
  hpf_theta_g_k_2 = hpf_theta_g_k_1;
  hpf_theta_g_k_1 = hpf_theta_g;
  
  return hpf_theta;
}


//-----------------------------------------------------------------------------
//      2*kp + ki
//  -----------------
//   s^2 + s*kp + ki
//  
float LpfTheta(float theta_a)
{
  static float theta_k_1;
  static float theta_k_2;
  static float theta_a_k_1;
  static float theta_a_k_2;
  
  float theta;
  
  
  theta = 1.9972*theta_k_1 - 0.9972*theta_k_2 + 0.0014*theta_a - 0.0014*theta_a_k_2;
  
  theta_k_2 = theta_k_1;
  theta_k_1 = theta;
  
  theta_a_k_2 = theta_a_k_1;
  theta_a_k_1 = theta_a;
  
  return theta;
}