#include <I2C.h>
#include "HMC5883L.h"


#define HMC5883L_ADDR  0x1E
#define HMC5883L_ID    0x48
#define RegCfgA        0x00
#define RegCfgB        0x01
#define RegMode        0x02
#define RegData        0x03
#define RegIden        0x0A

#define Measurement_Continuous 0x00
#define Measurement_SingleShot 0x01
#define Measurement_Idle       0x03



HMC5883L::HMC5883L()
{
  scaleFactor_ = 1;
}


void HMC5883L::Init()
{
  SetMeasurementMode(Measurement_Continuous);
}


void HMC5883L::ReadRaw(int *mx, int *my, int *mz)
{
  uint8_t* buffer = ReadRegister(RegData, 6);

  *mx = (buffer[0] << 8) | buffer[1];
  *my = (buffer[4] << 8) | buffer[5];
  *mz = (buffer[2] << 8) | buffer[3];
}


void HMC5883L::SetMeasurementMode(uint8_t mode)
{
	WriteRegister(RegMode, mode);
}


void HMC5883L::SetOutputRate(uint8_t hz)
{
  uint8_t regValue;


  regValue  = ReadRegister(RegCfgA, 1)[0];
  regValue &= 0xE3;              //  XXX0 00XX

  switch (hz) {
  case 3:
    regValue += 0x08;            //  XXX0 10XX
    WriteRegister(RegCfgA, regValue);
    break;

  case 15:
    regValue += 0x10;            //  XXX1 00XX
    WriteRegister(RegCfgA, regValue);
    break;

  case 30:
    regValue += 0x14;            //  XXX1 01XX
    WriteRegister(RegCfgA, regValue);
    break;

  case 75:
    regValue += 0x18;            //  XXX1 10XX
    WriteRegister(RegCfgA, regValue);
    break;

  default:
    break;
  }
} 


void HMC5883L::SetAvgSamples(uint8_t count)
{
  uint8_t regValue;


  regValue  = ReadRegister(RegCfgA, 1)[0];
  regValue &= 0x9F;              //  X00X XXXX

  switch (count) {
  case 1:
    WriteRegister(RegCfgA, regValue);
    break;

  case 2:
    regValue += 0x20;            // X01X XXXX
    WriteRegister(RegCfgA, regValue);
    break;

  case 4:
    regValue += 0x40;            // X10X XXXX
   WriteRegister(RegCfgA, regValue);
    break;

  case 8:
    regValue += 0x60;            // X11X XXXX
    WriteRegister(RegCfgA, regValue);
    break;

  default:
    break;
  }
}


bool HMC5883L::IsConnected()
{
	uint8_t data = ReadRegister(RegIden, 1)[0];

	if (HMC5883L_ID == data) {
		return true;
  } else {
		return false;
  }
}


uint8_t* HMC5883L::ReadRegister(int address, int length)
{
  uint8_t buffer[length];

  I2c.read(HMC5883L_ADDR, address, length, buffer);

  return buffer;
}


void HMC5883L::WriteRegister(int address, int value)
{
  I2c.write(HMC5883L_ADDR, address, value);
}