#include "NavCompass.h"


NavCompass::NavCompass()
{
  _magneto = new HMC5883L();

	_offsetX = 0;
	_offsetY = 0;
	_offsetZ = 0;
}


void NavCompass::init()
{
  _magneto->Init();
}


void NavCompass::update()
{
  int rawX;
  int rawY;
  int rawZ;


  _magneto->ReadRaw(&rawX, &rawY, &rawZ);

  _compX =  (rawX - _offsetX);
  _compY =  (rawY - _offsetY);
  _compZ =  (rawZ - _offsetZ);
}


float NavCompass::get_heading(float roll_rad, float pitch_rad)
{
  float mx;
  float my;

  float sinPhi;
  float cosPhi;
  float sinTheta;
  float cosTheta;


  sinPhi   = sin(roll_rad);
  cosPhi   = cos(roll_rad);
  sinTheta = sin(pitch_rad);
  cosTheta = cos(pitch_rad);

  mx = _compX*cosTheta + _compY*sinPhi*sinTheta + _compZ*cosPhi*sinTheta;
  my =                   _compY*cosPhi          - _compZ*sinPhi;

  return atan2(-my, mx);
}


void NavCompass::get_magnet(float *mx, float *my, float *mz)
{
  *mx = _compX;
  *my = _compY;
  *mz = _compZ;
}


void NavCompass::get_offsets(int *ox, int *oy, int *oz)
{
  *ox = _offsetX;
  *oy = _offsetY;
  *oz = _offsetZ;
}


void NavCompass::set_offsets(int ox, int oy, int oz)
{
  _offsetX = ox;
  _offsetY = oy;
  _offsetZ = oz;
}