#include <Arduino.h>
#include "NavIMU.h"


//-----------------------------------------------------------------------------
//
NavIMU::NavIMU(int cs_pin)
{
  _imu = new MPU6000(cs_pin);

  _ax = 0;
  _ay = 0;
  _az = 0;

  _p = 0;
  _q = 0;
  _r = 0;

  _p_bias = 0;
  _q_bias = 0;
  _r_bias = 0;
}


//-----------------------------------------------------------------------------
//
void
NavIMU::init()
{
  _imu->Init();
  delay(100);

  calc_gyro_bias();
}


//-----------------------------------------------------------------------------
//
void
NavIMU::update(void)
{
  static uint32_t prev_sample_time;

  int32_t sensors[7];
  

  _imu->Read(sensors);       // ax, ay, az, temp, p, q, r

  _ax =  0.0011975 * sensors[0];                   // [g], 9.81 / 8192
  _ay =  0.0011975 * sensors[1];
  _az =  0.0011975 * sensors[2];

  _p =  0.00106422 * (sensors[4] - _p_bias);       // [rad/s], 0.0174532 / 16.4
  _q =  0.00106422 * (sensors[5] - _q_bias);
  _r =  0.00106422 * (sensors[6] - _r_bias);


  _sample_time     = micros() - prev_sample_time;
  prev_sample_time = micros();
}


//-----------------------------------------------------------------------------
//
void 
NavIMU::get_gyro(float *p, float *q, float *r)
{
  *p = _p;
  *q = _q;
  *r = _r;
}


//-----------------------------------------------------------------------------
//
void 
NavIMU::get_accel(float *ax, float *ay, float *az)
{ 
  *ax = _ax;
  *ay = _ay;
  *az = _az;
} 


//-----------------------------------------------------------------------------
//
void
NavIMU::calc_gyro_bias()
{
  int32_t sensors[7];


  _p_bias = 0;
  _q_bias = 0;
  _r_bias = 0;

  for (int i = 1; i <= 100; i++) {
    _imu->Read(sensors);

    _p_bias += sensors[4];
    _q_bias += sensors[5];
    _r_bias += sensors[6];
  }

  _p_bias = _p_bias / 100;
  _q_bias = _q_bias / 100;
  _r_bias = _r_bias / 100;
}
